source('../R/growth_functions.R')


# first species 

experimental_growth_data=read.csv('data_vomitoria.csv', header=TRUE,dec=",",sep=";")
Benchmark.temp<-unique(experimental_growth_data$Temperature..C.)
K<-length(Benchmark.temp)
#experimental_growth_data<-experimental_growth_data$Temperature..C
##########################################################
# loading experimental growth data and corresponding times
##########################################################
exp.growthcurves = list()
exp.times = list()
for(kk in 1:length(Benchmark.temp)){
  ind<-which(experimental_growth_data[,2]==Benchmark.temp[kk])
  Lengths = experimental_growth_data[ind, 3]
  Times = experimental_growth_data[ind,1]
  #retained.size = length(Times)
  #Growthcurves = experimental_growth_data[!is.na(Lengths), 2 + (kk - 1) * 3]
  exp.times[[kk]] = Times
  exp.growthcurves[[kk]] = Lengths
}

L<-NULL
for(kk in 1:K){
  L<-c(L,max(exp.times[[kk]]))
}

G<-NULL
for(kk in 1:K){
  G<-c(G,max(exp.growthcurves[[kk]]))
}


vomitoria<-list(times=exp.times,growth.curves=exp.growthcurves,maxtimes=L,temperature=Benchmark.temp)


################  align species ###################

species<-list(species1=vomitoria)

llder<-vector("list", length(species))

for (s in 1:length(species)){
  
  llder[[s]] <- llderivgrowth(species[[s]]$growth.curves, species[[s]]$times, step = 1,df=8)
  
}

registered.benchmark.growth.curves<-vector("list",length(species))

for(s in 1:length(species)){ 
  registered.benchmark.growth.curves[[s]] = registered.growthcurves(llder[[s]]$Eval,llder[[s]]$estimated.growthcurves, nknots=10, norder=4, landmark0 = 0.7*species[[s]]$maxtimes[1])
}
############################


# bandwidth

h.bd<-4



# load temperature weather station

temp_weather_station = read.csv('temp_weather_station.csv', header=FALSE,sep=";")
temp_crime_scene =read.csv('temp_crime_scene.csv', header=FALSE,sep=";")

Temp.cs = rev(temp_crime_scene[,1])
Temp.ws = rev(temp_weather_station[,1])

plot(Temp.ws,type='l')
points(Temp.cs,type='l',col=3)

n = length(Temp.cs)
reslm = lm(Temp.cs ~ Temp.ws[2:(n+1)])
Predictions1 = reslm$coefficients[1] + reslm$coefficients[2] * Temp.ws[-(1:(n+1))]
# estimating temperatures at crime scene (with reversed time) via basic nonparametric model
Predictions2 = forecasting_crime_scene_temperatures(Temp.cs, Temp.ws, spar = 100)

# Set varying temperature curve
Temp_vary = round(rev(Predictions2),1) # you can try with Predictions2 and see the impact on the estimated post-mortem instant
#ts.plot(Temp_vary)
Times_temp_vary = 1:length(Temp_vary)

plot(1:length(temp_weather_station[,1])-length(Times_temp_vary), temp_weather_station[,1],type='l',xlab="Hours",ylab="Temperature",main="",cex.lab=2,cex.axis=2,lwd=2,ylim=c(4.5,28))
points((length(temp_weather_station[,1])-n+1):length(temp_weather_station[,1])-length(Times_temp_vary),temp_crime_scene[,1],type='l',col=4,lwd=2)
points(Times_temp_vary-length(Temp_vary),Temp_vary,type='l',col=4,lwd=2,lty=2)

Eval.temp = sort(unique(Temp_vary))



Observed.lengths<-read.csv('observed_vomitoria.csv', header=FALSE,sep=";")


tmin<-min(Times_temp_vary)
time.limits<-rep(NA,length(species))
for (s in 1:length(species)){
  index_t<-which.min((species[[s]]$temperature-tmin)^2)
  time.limits[s]<-max(species[[s]]$times[[index_t]])
}


res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp = sort(unique(Temp_vary)), 50, 4, mba=FALSE,min.temp=4.5,bandwidth = h.bd)
}



observed.lengths<-list(species1=Observed.lengths[,1])
times_temp_vary<-list(species1=Times_temp_vary)
temp_vary<-list(species1=Temp_vary)


Search<-seq(1, length(Times_temp_vary)-1, by = 1)
results.pmi<-growth_estimation(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp,  Search,postfeeding = FALSE)


results.pmi$pmi


# graphical output 
 

par(mfrow=c(2,2))

plot(Times_temp_vary-length(Times_temp_vary),Temp_vary,type='l',xlab="Hours",ylab="Temperature",main="Temperature profile",cex.lab=1.5,lwd=2)

const.temp<-constant_temp_growth(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp)

ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
xlim<-c(0,max(unlist(const.temp$EvalPoints)))
plot(0,0,col=0,xlim=xlim,ylim=ylim,ylab="length (mm)",xlab="hours",cex.lab=1.5,main="")
for (s in 1:length(observed.lengths)){
  for (k in 1:length(const.temp$GrowthCurves[[s]])){
    points(const.temp$EvalPoints[[s]][[k]],const.temp$GrowthCurves[[s]][[k]],type='l')
  }
}

time_axis<-Times_temp_vary[(length(Times_temp_vary)-results.pmi$pmi):length(Times_temp_vary)]-length(Times_temp_vary)
plot(time_axis,results.pmi$Growth.shape[[1]]$Length.progression,type='l',xlim=range(Times_temp_vary-length(Times_temp_vary)),ylim=ylim,lwd=3,xlab="hours",ylab="length (mm)",main="",cex.lab=2,cex.axis=2)
points(rep(time_axis[length(time_axis)],length(observed.lengths[[1]])),observed.lengths[[1]])
abline(v=Times_temp_vary[results.pmi$start.opt]-length(Times_temp_vary))


crit<-results.pmi$Criterion
crit[which(crit==Inf)]<-NA
crit<-approx(1:length(crit),crit,xout=1:length(crit))$y
taxis<-Times_temp_vary[Search]-length(Times_temp_vary)
tlim<-range(taxis[which(results.pmi$Criterion<Inf & results.pmi$Criterion>-Inf)])
plot(Times_temp_vary[Search]-length(Times_temp_vary),results.pmi$Criterion,type='l',xlim=tlim,col=4,lwd=3,xlab="Hours",ylab="criterion",cex.lab=2,cex.axis=2)
abline(v=Times_temp_vary[results.pmi$start.opt]-length(Times_temp_vary))


### Profile log-likelihood ####

obs.postfeeding <-FALSE

loglike<-rep(0,length(Search))
for (k in 1:length(Search)){ 
  growth2<-growth_shape(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp,  start=Search[k])
  
  for (s in 1:length(observed.lengths)){
    Lengths<-growth2$Growth.shapes[[s]]$Length.progression[length(growth2$Growth.shapes[[s]]$Length.progression)]
    if(is.na(Lengths)||growth2$Growth.shapes[[s]]$postfeeding!=obs.postfeeding){
      loglike[k]<-loglike[k]-Inf
    }else{
      loglike[k] = loglike[k] -sum((observed.lengths[[s]] - Lengths)^2)/(2*var(observed.lengths[[s]]))
    }
  }
}


# zoom
a<- -260
b<- -200
x<-Times_temp_vary[Search]-length(Times_temp_vary)
y<-loglike
y<- y[which(x>a & x< b)]
x<-x[which(x>a & x< b)]

plot(x,y,type='l',col=4,lwd=3,xlab="Hours",ylab="log-likelihood",cex.lab=2,cex.axis=2)
thr<-loglike[results.pmi$start.opt]-qchisq(.95,1)/2
abline(h=thr,lty=2,lwd=3)
CI<-x[which(y>=thr)]
abline(v=min(CI))
abline(v=max(CI))



########## Bayesian analysis ##########
library(MHadaptive)

## Gaussian prior

#mu_n<-150
#sigma_n<-2.5


#mu_n<<-length(Times_temp_vary)-220
#sigma_n<<-12
# n<-rnorm(1000,mu_n,sigma_n)
# tr<-density(n,bw="SJ",adjust=2)


# posterior<-function(N){
#   loglike<-0
#   growth2<-growth_shape(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp,  start=N)
#   
#   
#   for (s in 1:length(observed.lengths)){
#     Lengths<-growth2$Growth.shapes[[s]]$Length.progression[length(growth2$Growth.shapes[[s]]$Length.progression)]
#     if(is.na(Lengths)){
#       loglike<-loglike-Inf
#     }else{
#       loglike = loglike -sum((observed.lengths[[s]] - Lengths)^2)/(2*var(observed.lengths[[s]]))
#     }
#   }
#   post<-loglike-((N-mu_n)^2/(2*sigma_n^2))
#   
#   post
# }

# Uniform prior

unif.min<-length(Temp_vary)-230
unif.max<-length(Temp_vary)-11

x=unif.min:unif.max
tr<-list(x=x,y=rep(1/(unif.max-unif.min),length(x)))

posterior<-function(N){
  loglike<-0
  if(N>=unif.min & N<=unif.max){
    growth2<-growth_shape(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp,  start=N)


    for (s in 1:length(observed.lengths)){
      Lengths<-growth2$Growth.shapes[[s]]$Length.progression[length(growth2$Growth.shapes[[s]]$Length.progression)]
      if(is.na(Lengths)){
        loglike<-loglike-Inf
      }else{
      loglike = loglike -sum((observed.lengths[[s]] - Lengths)^2)/(2*var(observed.lengths[[s]]))
      }
    }

    post<-loglike+1
  }else{ post<--Inf}

  post
}





# Posterior sampling #


post<-Metro_Hastings(li_func=posterior,pars=c(200),burn_in=1000,iterations=1400)


# Plots # 

par(mfrow=c(1,2))
plot(tr$x-length(Times_temp_vary),tr$y,type='l',lwd=4,ylim=c(0,0.12),xlim=c(-350,0),xlab="Hatching time",ylab="Density",lty=2,col=4,cex.lab=1.5,cex.axis=1)

abline(v=mu_n,lty=2,col=4)


ptr<-density(post$trace,bw="SJ",adjust=20)
points(ptr$x-length(Times_temp_vary),ptr$y,type='l',lwd=4)
abline(v=mean(post$trace)-length(Times_temp_vary))


N<-mean(post$trace)

growth2<-growth_shape(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp,  start=ceiling(N))
const.temp<-constant_temp_growth(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp)

ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
time_axis<-Times_temp_vary[ceiling(N):length(Times_temp_vary)]-length(Times_temp_vary)
plot(time_axis,growth2$Growth.shapes[[1]]$Length.progression,type='l' ,xlim=range(Times_temp_vary)-length(Times_temp_vary),ylim=ylim,lwd=3,xlab="hours",ylab="length (mm)",main="",cex.lab=1.5)
points(rep(time_axis[length(time_axis)],length(observed.lengths[[1]])),observed.lengths[[1]])
abline(v=mean(post$trace)-length(Times_temp_vary))

