source('../R/growth_functions.R')

def_par<-par(no.readonly = TRUE)

## C. vicina esperimental data plot ####

experimental_growth_data = read.csv('../ID303/length_data.csv', header=TRUE,sep=",",dec=".")
experimental_growth_data = experimental_growth_data[, -(1:3)]

# experimental temperatures

Benchmark.temp = c(4.5, 7.5, 10.0, 12.5, 15.0, 18.0, 21.0, 24.0, 27.0)


##########################################################
# loading experimental growth data and corresponding times
##########################################################
K = length(Benchmark.temp)
exp.growthcurves = list()
exp.times = list()
for(kk in 1:K){
  Lengths = experimental_growth_data[, 2 + (kk - 1) * 3]
  Times = experimental_growth_data[!is.na(Lengths), 1 + (kk - 1) * 3]
  retained.size = length(Times)
  Growthcurves = experimental_growth_data[!is.na(Lengths), 2 + (kk - 1) * 3]
  exp.times[[kk]] = Times[1:retained.size]
  exp.growthcurves[[kk]] = Growthcurves[1:retained.size]
}



par(mfrow=c(3,3), mar = c(4.5,5,3,1), cex.lab = 2,cex.main=2.5,cex.axis=1.5)
xlim = range(exp.times)
ylim = c(0,24)
for(kk in 1:K){
  #plot(exp.times[[kk]], exp.growthcurves[[kk]], xlim = xlim, ylim = ylim, xlab = "hours", ylab = "lengths (mm)", main = paste(expression(theta), kk, "=", Benchmark.temp[kk], "degrees (C)", sep = " "))  
  plot(exp.times[[kk]], exp.growthcurves[[kk]], xlim = xlim, ylim = ylim, xlab = "hours", ylab = "lengths (mm)", main = paste("T=", Benchmark.temp[kk], "°C", sep = " "))  
  
}

# grid plot 
cols<-rainbow(n=9,start=0.55,end=0)
par(mfrow = c(3, 3))
par(cex = 0.6,cex.axis=2)
par(mar = c(0, 0, 0, 0), oma = c(6, 6, 0.5, 1))
par(tcl = -0.25)
par(mgp = c(2, 0.6, 0))
xlim = range(exp.times)
ylim = c(0,23)
for (kk in 1:K) {
  plot(exp.times[[kk]], exp.growthcurves[[kk]], pch=18 , xlim = xlim, ylim = ylim, xlab = "hours", ylab = "lengths (mm)",axes = FALSE,col = cols[kk])  
  mtext(paste("T=", Benchmark.temp[kk], "°C", sep = " "), side = 3, line = -2.2, adj = 0.95, cex = 1.5, col = 1)
  if (kk %in% c(7, 8, 9))
  axis(1, col = "grey40", col.axis = "grey20", at = seq(0,1300,by=200))
  if (kk %in% c(1, 4,7))
  axis(2, col = "grey40", col.axis = "grey20", at = seq(0,21,by=3))
  box(col = "grey60")
}
mtext("hours", side = 1, outer = TRUE, cex = 2, line = 3.5, col = 1)
mtext("length (mm)", side = 2, outer = TRUE, cex = 2, line = 3,col =1)








# reset to default graphical parameters
par(def_par)
#windows()

par(mar = c(4.5,5,3,1), cex.lab = 2,cex.main=2.5,cex.axis=1.5)

xlim = range(exp.times)
ylim = range(exp.growthcurves)
plot(exp.times[[1]], exp.growthcurves[[1]], xlim = xlim, ylim = c(0,40),col=cols[kk], xlab = "hours", ylab = "lengths (mm)",lwd=3,cex.lab=1.5,cex.axis=1.5)  

for(kk in 1:(K)){
  #plot(exp.times[[kk]], exp.growthcurves[[kk]], xlim = xlim, ylim = ylim, xlab = "hours", ylab = "lengths (mm)", main = paste(expression(theta), kk, "=", Benchmark.temp[kk], "degrees (C)", sep = " "))  
  points(exp.times[[kk]], exp.growthcurves[[kk]], xlim = xlim, ylim = ylim, xlab = "hours", ylab = "lengths (mm)", col=cols[kk])
  
}

llder<-vector("list", length(species))

for (s in 1:length(species)){
  
  llder[[s]] <- llderivgrowth(species[[s]]$growth.curves, species[[s]]$times, step = 1)
  
}


par(mar = c(4.5,5,3,1), cex.lab = 2,cex.main=2.5,cex.axis=2)

xlim = range(llder[[1]]$Eval)
ylim.growthcurves = c(-1,19)
cols<-rainbow(n=9,start=0.55,end=0)
plot(llder[[1]]$Eval[[1]], llder[[1]]$estimated.growthcurves[[1]], col=cols[1],xlab = "hours since hatching", ylab = "lengths (mm)", type ="l", xlim = xlim, ylim = ylim.growthcurves, main = "",lwd=3)
lg<-paste0("T = ",Benchmark.temp[1],"°C")

if(K > 1){
  for(kk in 2:K){
    
    points(llder[[1]]$Eval[[kk]], llder[[1]]$estimated.growthcurves[[kk]], type ="l",lwd=3,lty=kk,col=cols[kk])
    lg<-c(lg,paste0("T = ",Benchmark.temp[kk],"°C"))
  }
}

legend(x=800,y=7,lty=1:K,legend = lg,ncol=2,col=cols,lwd=3)
# reset to default graphical parameters
par(pardef)


par(mfrow=c(1,2), mar = c(5,4.5,2,1), cex.lab = 1.5)


xlim = range(registered.benchmark.growth.curves[[1]]$Eval/max(registered.benchmark.growth.curves[[1]]$Eval.points))
ylim.growthcurves = range(registered.benchmark.growth.curves[[1]]$ALIGNED.GROWTH.CURVES) 
#xlim = range(llder[[1]]$Eval)
#ylim.growthcurves = c(-1,19)
cols<-rainbow(n=9,start=0.55,end=0)


plot(registered.benchmark.growth.curves[[1]]$Eval.points/max(registered.benchmark.growth.curves[[1]]$Eval.points), registered.benchmark.growth.curves[[1]]$ALIGNED.GROWTH.CURVES[,1], col=cols[1],xlab = "Transformed time scale", ylab = "Length (mm)", type ="l", xlim = xlim, ylim = ylim.growthcurves, main = "",lwd=3,cex.lab=1.5)
if(K > 1){
  for(kk in 2:K){
    
    points(registered.benchmark.growth.curves[[1]]$Eval.points/max(registered.benchmark.growth.curves[[1]]$Eval.points), registered.benchmark.growth.curves[[1]]$ALIGNED.GROWTH.CURVES[,kk], type ="l",lwd=3,lty=kk,col=cols[kk])
  }
}

legend(x=0.5,y=9.8,lty=1:K,legend = lg,ncol=1,col=cols,lwd=3)

xlim = range(registered.benchmark.growth.curves[[1]]$Eval)
ylim.growthcurves = range(registered.benchmark.growth.curves[[1]]$WARPED.TIME)/max(registered.benchmark.growth.curves[[1]]$Eval.points) 
plot(registered.benchmark.growth.curves[[1]]$Eval.points, registered.benchmark.growth.curves[[1]]$WARPED.TIME[,1]/max(registered.benchmark.growth.curves[[1]]$Eval.points),col=cols[1], xlab = "Hours since hatching", ylab = "Warping functions", type ="l", xlim = xlim, ylim = ylim.growthcurves, main = "",lwd=3,cex.lab=1.5)
if(K > 1){
  for(kk in 2:K){
    
    points(registered.benchmark.growth.curves[[1]]$Eval.points, registered.benchmark.growth.curves[[1]]$WARPED.TIME[,kk]/max(registered.benchmark.growth.curves[[1]]$Eval.points), type ="l",lwd=3,lty=kk,col=cols[kk])
  }
}





########## estimated growth curves and empirical ones ################
par(mfrow=c(1,1), mar = c(5,4.5,2,1), cex.lab = 2,cex.axis=2)

const.temp<-constant_temp_growth(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp)

xlim = range(llder[[1]]$Eval)
ylim = c(-1,21)


plot(0,0,col=0,xlim=xlim,ylim=ylim,ylab="Length (mm)",xlab="Hours",main="")
for (s in 1:length(observed.lengths)){
  for (k in 1:length(const.temp$GrowthCurves[[s]])){
    points(const.temp$EvalPoints[[s]][[k]],const.temp$GrowthCurves[[s]][[k]],type='l')
  }
}


lg<-c()
K<-length(llder[[1]]$estimated.growthcurves)
cols<-rainbow(n=9,start=0.55,end=0)

for(kk in 1:K){
    
    points(llder[[1]]$Eval[[kk]], llder[[1]]$estimated.growthcurves[[kk]], type ="l",lwd=3,lty=kk+1,col=cols[kk])
    lg<-c(lg,paste0("T = ",Benchmark.temp[kk]," C"))
}


legend(x=800,y=8,lty=2:(K+1),legend = lg,ncol=2,col=cols,lwd=3)


par(mar=c(5,5,4,1),cex.lab=2,cex.axis=2,cex.main=2)

plot(Times_temp_vary-length(Times_temp_vary),Temp_vary,type='l',xlab="Hours",ylab="Temperature (°C)",main="Temperature profile",lwd=2,col=4)


time_axis<-Times_temp_vary[(length(Times_temp_vary)-results.pmi$pmi):length(Times_temp_vary)]-length(Times_temp_vary)
plot(time_axis,results.pmi$Growth.shape[[1]]$Length.progression,type='l',xlim=range(Times_temp_vary-length(Times_temp_vary)),ylim=ylim,lwd=3,xlab="Hours",ylab="Length (mm)",main="Estimated larval growth curve")
points(rep(time_axis[length(time_axis)],length(observed.lengths[[1]])),observed.lengths[[1]])
abline(v=Times_temp_vary[results.pmi$start.opt]-length(Times_temp_vary))


taxis<-Times_temp_vary[Search]-length(Times_temp_vary)
tlim<-range(taxis[which(results.pmi$Criterion<Inf & results.pmi$Criterion>-Inf)])
plot(Times_temp_vary[Search]-length(Times_temp_vary),results.pmi$Criterion,type='l',xlim=tlim,col=4,lwd=3,xlab="Hours",ylab="Criterion")
abline(v=Times_temp_vary[results.pmi$start.opt]-length(Times_temp_vary))



## plot log-likelihood ##

par(mar=c(5,5,4,1),cex.lab=2,cex.axis=2,cex.main=2)

plot(Times_temp_vary[Search]-length(Times_temp_vary),loglike,type='l',col=4,lwd=3,xlab="Hours",ylab="Log-likelihood")

thr<-loglike[results.pmi$start.opt]-qchisq(.95,1)/2
abline(h=thr,lty=2,lwd=3)
CI<-x[which(y>thr)]
abline(v=min(CI))
abline(v=max(CI))



par(mfrow=c(1,2),mar=c(5,5,4,1),cex.lab=2,cex.axis=2,cex.main=2)
plot(tr$x-length(Times_temp_vary),tr$y,type='l',lwd=4,ylim=c(0,0.025),xlim=c(-401,0),xlab="Hatching time",ylab="Density",lty=2,col=4)

abline(v=mu_n,lty=2,col=4)


### posterior distributions ##

ptr<-density(post$trace,bw="SJ",adjust=20)
points(ptr$x-length(Times_temp_vary),ptr$y,type='l',lwd=4)
abline(v=mean(post$trace)-length(Times_temp_vary))


N<-mean(post$trace)

growth2<-growth_shape(observed.lengths, Times_temp_vary, Temp_vary, res.agpaact, Eval.temp,  start=ceiling(N))
time_axis<-Times_temp_vary[ceiling(N):length(Times_temp_vary)]-length(Times_temp_vary)
plot(time_axis,growth2$Growth.shapes[[1]]$Length.progression,type='l' ,xlim=range(Times_temp_vary)-length(Times_temp_vary),ylim=ylim,lwd=3,xlab="Hours",ylab="Length (mm)",main="")
points(rep(time_axis[length(time_axis)],length(observed.lengths[[1]])),observed.lengths[[1]])
abline(v=mean(post$trace)-length(Times_temp_vary))

par(mfrow=c(1,1), mar = c(5,4.5,2,1), cex.lab = 2,cex.axis=2)

plot(1:length(temp_weather_station[,1])-length(Times_temp_vary), temp_weather_station[,1],type='l',xlab="Hours",ylab="Temperature (°C)",main="",lwd=2,ylim=c(4.5,28))
points((length(temp_weather_station[,1])-n+1):length(temp_weather_station[,1])-length(Times_temp_vary),temp_crime_scene[,1],type='l',col=4,lwd=2)
points(Times_temp_vary-length(Temp_vary),Temp_vary,type='l',col=4,lwd=2,lty=2)
