source('growth_functions.R')

# import experimental data

experimental_growth_data = read.csv('../ID303/length_data.csv', header=TRUE,sep=",",dec=".")
experimental_growth_data = experimental_growth_data[, -(1:3)]

# experimental temperatures

Benchmark.temp = c(4.5, 7.5, 10.0, 12.5, 15.0, 18.0, 21.0, 24.0, 27.0)


##########################################################
# loading experimental growth data and corresponding times
##########################################################
K = length(Benchmark.temp)
exp.growthcurves = list()
exp.times = list()
for(kk in 1:K){
  Lengths = experimental_growth_data[, 2 + (kk - 1) * 3]
  Times = experimental_growth_data[!is.na(Lengths), 1 + (kk - 1) * 3]
  retained.size = length(Times)
  Growthcurves = experimental_growth_data[!is.na(Lengths), 2 + (kk - 1) * 3]
  exp.times[[kk]] = Times[1:retained.size]
  exp.growthcurves[[kk]] = Growthcurves[1:retained.size]
}


L<-NULL
for(kk in 1:K){
  L<-c(L,max(exp.times[[kk]]))
}

G<-NULL
for(kk in 1:K){
  G<-c(G,max(exp.growthcurves[[kk]]))
}


vicina<-list(times=exp.times,growth.curves=exp.growthcurves,maxtimes=L,temperature=Benchmark.temp)


################  align species ###################

species<-list(species1=vicina)

llder<-vector("list", length(species))

for (s in 1:length(species)){
  
  llder[[s]] <- llderivgrowth(species[[s]]$growth.curves, species[[s]]$times, step = 1)
  
}

registered.benchmark.growth.curves<-vector("list",length(species))

for(s in 1:length(species)){ 
  registered.benchmark.growth.curves[[s]] = registered.growthcurves(llder[[s]]$Eval,llder[[s]]$estimated.growthcurves, nknots=10, norder=4, landmark0 = 0.7*species[[s]]$maxtimes[1])
}
############################


#### bandwidth: ###

h.bd<-4



########### simulated example 1 #############

#### constant temperature + noise #####

## feeding 

true_temp<-rep(10,200)
times<-1:200
obs.species<-list(NULL)
Eval.temp = sort(unique(true_temp))

res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp , 50, 4, mba=FALSE,bandwidth=h.bd)
}



pmi<-100
growth2<-growth_shape(observed.lengths=obs.species, Times_temp_vary=times, Temp_vary=true_temp, res.agpaact=res.agpaact, Eval.temp=Eval.temp,  start=length(times)-pmi)

exp_sd<-0.1

const.temp<-constant_temp_growth(obs.species, times, true_temp, res.agpaact, Eval.temp)
ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
time_axis<-times[(length(times)-pmi):length(times)]
plot(time_axis,growth2$Growth.shapes[[1]]$Length.progression,type='l' ,xlim=range(times),ylim=ylim,lwd=3,xlab="hours",ylab="length (mm)",main=paste("Hatching time =",-pmi," hours"),cex.lab=1.3)

#observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi]))

#Temp_vary<-rnorm(400,16,1)
#plot(Temp_vary,type='l',ylim=c(10,20))

#sigma<-c(0.1,0.25,0.75,1)
sigma<-c(0.1,0.25,0.75,1)
reps<-1000
sim_pmi<-matrix(NA,reps,length(sigma))

for ( sig in 1:length(sigma)){ 
for (k in 1:reps){
  observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi],sd=exp_sd))
Temp_vary<-rnorm(length(times),mean(true_temp),sigma[sig])
Eval.temp = sort(unique(Temp_vary))

res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp = sort(unique(Temp_vary)), 50, 4, mba=FALSE,bandwidth=h.bd)
}

Search<-seq(3, length(times)-1, by = 1)
sim_pmi[k,sig]<- -growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search,postfeeding = FALSE)$pmi
print(paste("-- rep=",k,",sd=",sigma[sig]))

}
}


results.pmi<-growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search, postfeeding = FALSE)





hist(sim_pmi[,1],prob=TRUE,xlim=c(-110,-90),breaks=6,ylim=c(0,0.6),xlab="Hatching time",main=expression(paste(sigma[T],"=0.1")),col="gray")
abline(v=-100,lwd=3,col=4)
hist(sim_pmi[,2],prob=TRUE,xlim=c(-110,-90),breaks=6,ylim=c(0,0.6),xlab="Hatching time",main=expression(paste(sigma[T],"=0.25")),col="gray")
abline(v=-100,lwd=3,col=4)
hist(sim_pmi[,3],prob=TRUE,xlim=c(-110,-90),breaks=6,ylim=c(0,0.6),xlab="Hatching time",main=expression(paste(sigma[T],"=0.75")),col="gray")
abline(v=-100,lwd=3,col=4)
hist(sim_pmi[,4],prob=TRUE,xlim=c(-110,-90),breaks=6,ylim=c(0,0.8),xlab="Hatching time",main=expression(paste(sigma[T],"=1")),col="gray")
abline(v=-100,lwd=3,col=4)



########### simulated example 2 #############

################ real temperature ###################

temp_weather_station<-read.csv('../ID307/temp_weather_station.csv', header=FALSE,sep=";")
true_temp<-temp_weather_station[1:200,1]

times<-1:length(true_temp)
obs.species<-list(NULL)
Eval.temp = sort(unique(true_temp))


res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp , 50, 4, mba=FALSE,bandwidth=h.bd)
}



pmi<-100
growth2<-growth_shape(observed.lengths=obs.species, Times_temp_vary=times, Temp_vary=true_temp, res.agpaact=res.agpaact, Eval.temp=Eval.temp,  start=length(times)-pmi)

exp_sd<-0.7

const.temp<-constant_temp_growth(obs.species, times, true_temp, res.agpaact, Eval.temp)
ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
time_axis<-times[(length(times)-pmi):length(times)]
plot(time_axis,growth2$Growth.shapes[[1]]$Length.progression,type='l' ,xlim=range(times),ylim=ylim,lwd=3,xlab="hours",ylab="length (mm)",main=paste("Hatching time =",-pmi," hours"),cex.lab=1.3)


sigma<-c(0.1,0.25,0.75,1)
reps<-1000
sim_pmi<-matrix(NA,reps,length(sigma))

for ( sig in 1:length(sigma)){ 
  for (k in 1:reps){
    observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi],sd=exp_sd))
    Temp_vary<-true_temp+rnorm(length(true_temp),0,sigma[sig])
    Eval.temp = sort(unique(Temp_vary))
    
    res.agpaact<-vector("list",length(species))
    
    for (s in 1:length(species)){ 
      res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp = sort(unique(Temp_vary)), 50, 4, mba=FALSE,bandwidth=h.bd)
    }
    
    Search<-seq(3, length(times)-1, by = 1)
    sim_pmi[k,sig]<- -growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search,postfeeding = FALSE)$pmi
    print(paste("-- rep=",k,",sd=",sigma[sig]))
    
  }
}



par(mar=c(5, 4.5, 4, 2) + 0.1)
plot(times,Temp_vary,type='l',xlab="Hours",ylab="Temperature",main="",cex.lab=2,lwd=2,col=4,cex.axis=2)
points(times,true_temp,lwd=2,lty=2,type='l')



par(mfrow=c(2,2),mar=c(5,5,4,1),cex.lab=2,cex.axis=2,cex.main=2,lwd=2)


hist(sim_pmi[,1],prob=TRUE,xlim=c(-110,-90),ylim=c(0,0.4),xlab="Hatching time",main=expression(paste(sigma[T],"=0.1")),col="gray")
abline(v=-100,lwd=3,col=4)
hist(sim_pmi[,2],prob=TRUE,xlim=c(-110,-90),ylim=c(0,0.4),xlab="Hatching time",main=expression(paste(sigma[T],"=0.25")),col="gray")
abline(v=-100,lwd=3,col=4)
hist(sim_pmi[,3],prob=TRUE,xlim=c(-110,-90),ylim=c(0,0.4),xlab="Hatching time",main=expression(paste(sigma[T],"=0.75")),col="gray")
abline(v=-100,lwd=3,col=4)
hist(sim_pmi[,4],prob=TRUE,xlim=c(-110,-90),ylim=c(0,0.4),xlab="Hatching time",main=expression(paste(sigma[T],"=1")),col="gray")
abline(v=-100,lwd=3,col=4)








########### simulated example 3 #############

#### constant temperature + noise and post-feeding region#####

true_temp<-rep(15,400)
times<-1:400
obs.species<-list(NULL)
Eval.temp = sort(unique(true_temp))

res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp , 50, 4, mba=FALSE,bandwidth=h.bd)
}



pmi<-320
growth2<-growth_shape(observed.lengths=obs.species, Times_temp_vary=times, Temp_vary=true_temp, res.agpaact=res.agpaact, Eval.temp=Eval.temp,  start=length(times)-pmi,postfeeding = TRUE)

exp_sd<-0.1

const.temp<-constant_temp_growth(obs.species, times, true_temp, res.agpaact, Eval.temp)
ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
time_axis<-times[(length(times)-pmi):length(times)]
plot(time_axis,growth2$Growth.shapes[[1]]$Length.progression,type='l' ,xlim=range(times),ylim=ylim,lwd=3,xlab="hours",ylab="length (mm)",main=paste("Hatching time =",-pmi," hours"),cex.lab=1.3)
points(rep(time_axis[length(time_axis)],20),rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi],sd=exp_sd))

#observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi]))

#Temp_vary<-rnorm(400,16,1)
#plot(Temp_vary,type='l',ylim=c(10,20))

#sigma<-c(0.1,0.25,0.75,1)
sigma<-c(0.1,0.25,0.75,1)
reps<-1000
sim_pmi<-matrix(NA,reps,length(sigma))

for ( sig in 1:length(sigma)){ 
  for (k in 1:reps){
    observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi],sd=exp_sd))
    Temp_vary<-rnorm(length(times),mean(true_temp),sigma[sig])
    Eval.temp = sort(unique(Temp_vary))
    
    res.agpaact<-vector("list",length(species))
    
    for (s in 1:length(species)){ 
      res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp = sort(unique(Temp_vary)), 50, 4, mba=FALSE,bandwidth=h.bd)
    }
    
    Search<-seq(50, length(times)-50, by = 1)
    sim_pmi[k,sig]<- -growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search,postfeeding = TRUE)$pmi
    print(paste("-- rep=",k,",sd=",sigma[sig]))
    
  }
  save(sim_pmi,sig,file="../../Sim results V4/sim_post_01.RData")
}


results.pmi<-growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search, postfeeding = TRUE)




par(mfrow=c(2,2),mar= c(2, 3, 2, 1))
hist(sim_pmi[,1],prob=TRUE,xlim=c(-400,-150),breaks=8,ylim=c(0,0.03),xlab="Hatching time",main=expression(paste(sigma[T],"=0.1")),col="gray")
abline(v=-320,lwd=3,col=4)
hist(sim_pmi[,2],prob=TRUE,xlim=c(-400,-150),breaks=8,ylim=c(0,0.03),xlab="Hatching time",main=expression(paste(sigma[T],"=0.25")),col="gray")
abline(v=-320,lwd=3,col=4)
hist(sim_pmi[,3],prob=TRUE,xlim=c(-400,-150),breaks=8,ylim=c(0,0.03),xlab="Hatching time",main=expression(paste(sigma[T],"=0.75")),col="gray")
abline(v=-320,lwd=3,col=4)
hist(sim_pmi[,4],prob=TRUE,xlim=c(-400,-150),breaks=8,ylim=c(0,0.03),xlab="Hatching time",main=expression(paste(sigma[T],"=1")),col="gray")
abline(v=-320,lwd=3,col=4)

