source('growth_functions.R')
library(readxl)
library(mgcv)

experimental_growth_data=read.csv('../ID307/data_vomitoria.csv', header=TRUE,dec=",",sep=";")
Benchmark.temp<-unique(experimental_growth_data$Temperature..C.)
K<-length(Benchmark.temp)

##########################################################
# loading experimental growth data and corresponding times
##########################################################

exp.growthcurves = list()
exp.times = list()
for(kk in 1:length(Benchmark.temp)){
  ind<-which(experimental_growth_data[,2]==Benchmark.temp[kk])
  Lengths = experimental_growth_data[ind, 3]
  Times = experimental_growth_data[ind,1]
  exp.times[[kk]] = Times
  exp.growthcurves[[kk]] = Lengths
}

L<-NULL
for(kk in 1:K){
  L<-c(L,max(exp.times[[kk]]))
}

G<-NULL
for(kk in 1:K){
  G<-c(G,max(exp.growthcurves[[kk]]))
}


vomitoria<-list(times=exp.times,growth.curves=exp.growthcurves,maxtimes=L,temperature=Benchmark.temp)




################  align species ###################

species<-list(species1=vomitoria)

llder<-vector("list", length(species))

for (s in 1:length(species)){
  
  llder[[s]] <- llderivgrowth(species[[s]]$growth.curves, species[[s]]$times, step = 1,df=8)
  
}

registered.benchmark.growth.curves<-vector("list",length(species))

for(s in 1:length(species)){ 
  registered.benchmark.growth.curves[[s]] = registered.growthcurves(llder[[s]]$Eval,llder[[s]]$estimated.growthcurves, nknots=10, norder=4, landmark0 = 0.7*species[[s]]$maxtimes[1])
}
############################


##### ADH model fit ####

C_vomitoria_Length_data <- read_excel("../ID307/C. vomitoria Length data.xls")
adh_model<-gam(ADH~ s(Length),data = C_vomitoria_Length_data)
attach(C_vomitoria_Length_data)
plot(Length,ADH,cex.lab=2,cex.axis=2)
ix<-sort(Length,index=TRUE)$ix
points(Length[ix],adh_model$fitted.values[ix],type='l',lwd=2)
detach(C_vomitoria_Length_data)



#### bandwidth: ####

h.bd<-4


########### simulated scenario 1 #############



#### constant temperature + noise #####


true_temp<-rep(12,200)
times<-1:200
obs.species<-list(NULL)
Eval.temp = sort(unique(true_temp))


res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp , 50, 4, mba=FALSE,bandwidth=h.bd)
}



pmi0<-100
growth2<-growth_shape(observed.lengths=obs.species, Times_temp_vary=times, Temp_vary=true_temp, res.agpaact=res.agpaact, Eval.temp=Eval.temp,  start=length(times)-pmi0)

exp_sd<-0.7

const.temp<-constant_temp_growth(obs.species, times, true_temp, res.agpaact, Eval.temp)
ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
time_axis<-times[(length(times)-pmi0):length(times)]


sigma<-c(0.1,0.25,0.75,1)
reps<-1000
sim_pmi<-matrix(NA,reps,length(sigma))
sim_adh_pmi<-matrix(NA,reps,length(sigma))
for ( sig in 1:length(sigma)){ 
  for (k in 1:reps){
    observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi0],sd=exp_sd))
    Temp_vary<-rnorm(length(times),mean(true_temp),sigma[sig])
    Eval.temp = sort(unique(Temp_vary))
    
    res.agpaact<-vector("list",length(species))
    
    for (s in 1:length(species)){ 
      res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp = sort(unique(Temp_vary)), 50, 4, mba=FALSE,bandwidth=h.bd)
    }
    
    Search<-seq(20, length(times)-20, by = 1)
    sim_pmi[k,sig]<- -growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search,postfeeding = FALSE)$pmi
    
    sub_length<-observed.lengths$species1
    star_length<-exp(mean(log(sub_length)))
    adh<-predict(adh_model,newdata = data.frame(Length=star_length))
    pmi<-which(cumsum(rev(Temp_vary-4.3))>as.double(adh))[1]
    sim_adh_pmi[k,sig]<- -pmi
    
    print(paste("-- rep=",k,",sd=",sigma[sig]))
    
  }
}




########### simulated scenario 2 ########### 

#### real temperature profile with random error


temp_weather_station<-read.csv('../ID307/temp_weather_station.csv', header=FALSE,sep=";")
true_temp<-temp_weather_station[1:200,1]

times<-1:length(true_temp)
obs.species<-list(NULL)
Eval.temp = sort(unique(true_temp))


res.agpaact<-vector("list",length(species))

for (s in 1:length(species)){ 
  res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp , 50, 4, mba=FALSE,bandwidth=h.bd)
}



pmi0<-100
growth2<-growth_shape(observed.lengths=obs.species, Times_temp_vary=times, Temp_vary=true_temp, res.agpaact=res.agpaact, Eval.temp=Eval.temp,  start=length(times)-pmi0)

exp_sd<-0.7

const.temp<-constant_temp_growth(obs.species, times, true_temp, res.agpaact, Eval.temp)
ylim<-c(0,max(unlist(const.temp$GrowthCurves)))+2
time_axis<-times[(length(times)-pmi0):length(times)]
plot(time_axis,growth2$Growth.shapes[[1]]$Length.progression,type='l' ,xlim=range(times),ylim=ylim,lwd=3,xlab="hours",ylab="length (mm)",main=paste("Hatching time =",-pmi0," hours"),cex.lab=1.3)

sigma<-c(0.1,1)
reps<-1000
sim_pmi<-matrix(NA,reps,length(sigma))
sim_adh_pmi<-matrix(NA,reps,length(sigma))
for ( sig in 1:length(sigma)){ 
  for (k in 1:reps){
    observed.lengths<-list(species1=rnorm(20,growth2$Growth.shapes[[1]]$Length.progression[pmi0],sd=exp_sd))
    Temp_vary<-true_temp+rnorm(length(true_temp),0,sigma[sig])
    Eval.temp = sort(unique(Temp_vary))
    
    res.agpaact<-vector("list",length(species))
    
    for (s in 1:length(species)){ 
      res.agpaact[[s]] = approx.GP.reg(llder[[s]]$Eval,registered.benchmark.growth.curves[[s]], species[[s]]$temperature, Eval.temp = sort(unique(Temp_vary)), 50, 4, mba=FALSE,bandwidth=h.bd)
    }
    
    Search<-seq(3, length(times)-1, by = 1)
    sim_pmi[k,sig]<- -growth_estimation(observed.lengths, times, Temp_vary, res.agpaact, Eval.temp,  Search,postfeeding = FALSE)$pmi
   
    sub_length<-observed.lengths$species1[observed.lengths$species1>=quantile(observed.lengths$species1,0.25)]
    star_length<-exp(mean(log(sub_length)))
    adh<-predict(adh_model,newdata = data.frame(Length=star_length))
    pmi<-which(cumsum(rev(Temp_vary-4.3))>as.double(adh))[1]
    sim_adh_pmi[k,sig]<- -pmi
    print(paste("-- rep=",k,",sd=",sigma[sig]))
     
    
  }
}


## boxplots ##

k<-1
plot(0,0,xlim=c(0.75,5.5),ylim=c(-110,-75),col=0,xlab="",ylab = "",axes = FALSE)
for (sig in 1:length(sigma)){
  boxplot(sim_adh_pmi[,sig],sim_pmi[,sig],ann = FALSE,add=TRUE,xlab=expression(sigma),at=k:(k+1),col=c("white","gray"),xaxt='n')
  k<-k+3
}
abline(h=-100)
abline(v=3,lty=2)


axis(1,at=c(1.5,4.5),labels=sigma,cex.axis=2.5)
mtext(side=1,text=expression(sigma),line=3,cex = 3)
mtext(side=2,text="Estimated hatching time (hrs)",line=3,cex = 2)

